# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

from unittest import TestCase

import dask.dataframe as dd
import pandas as pd

from tsfresh.examples.driftbif_simulation import load_driftbif
from tsfresh import extract_relevant_features, extract_features
from tsfresh.feature_extraction import MinimalFCParameters


class FeatureExtractionTestCase(TestCase):
    def setUp(self):
        df, y = load_driftbif(100, 10, classification=True, seed=42)

        df['my_id'] = df['id'].astype('str')
        del df["id"]

        self.df = df

    def test_pandas(self):
        df = self.df

        # Test shape and a single entry (to see if it works at all)
        X = extract_features(df, column_id="my_id", column_sort="time", column_kind="dimension", column_value="value",
                             default_fc_parameters=MinimalFCParameters())
        self.assertIn("1__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "1__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 16))

        X = extract_features(df, column_id="my_id", column_sort="time", column_kind="dimension",
                             default_fc_parameters=MinimalFCParameters())
        self.assertIn("1__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "1__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 16))

        X = extract_features(df.drop(columns=["dimension"]), column_id="my_id", column_sort="time",
                             default_fc_parameters=MinimalFCParameters())
        self.assertIn("value__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "value__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 8))

        X = extract_features(df.drop(columns=["dimension", "time"]), column_id="my_id",
                             default_fc_parameters=MinimalFCParameters())
        self.assertIn("value__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "value__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 8))

    def test_pandas_no_pivot(self):
        df = self.df

        X = extract_features(df, column_id="my_id", column_sort="time",
                             column_kind="dimension", column_value="value",
                             pivot=False,
                             default_fc_parameters=MinimalFCParameters())
        X = pd.DataFrame(X, columns=["my_id", "variable", "value"])
        self.assertIn("1__mean", X["variable"].values)
        self.assertAlmostEqual(X[(X["my_id"] == "5") & (X["variable"] == "1__mean")]["value"].iloc[0], 5.516e-05, 4)
        self.assertEqual(X.shape, (100*16, 3))

        X = extract_features(df, column_id="my_id", column_sort="time",
                             column_kind="dimension",
                             pivot=False,
                             default_fc_parameters=MinimalFCParameters())
        X = pd.DataFrame(X, columns=["my_id", "variable", "value"])
        self.assertIn("1__mean", X["variable"].values)
        self.assertAlmostEqual(X[(X["my_id"] == "5") & (X["variable"] == "1__mean")]["value"].iloc[0], 5.516e-05, 4)
        self.assertEqual(X.shape, (100*16, 3))

        X = extract_features(df.drop(columns=["dimension"]), column_id="my_id",
                             column_sort="time",
                             pivot=False,
                             default_fc_parameters=MinimalFCParameters())
        X = pd.DataFrame(X, columns=["my_id", "variable", "value"])
        self.assertIn("value__mean", X["variable"].values)
        self.assertAlmostEqual(X[(X["my_id"] == "5") & (X["variable"] == "value__mean")]["value"].iloc[0], 5.516e-05, 4)
        self.assertEqual(X.shape, (100*8, 3))

        X = extract_features(df.drop(columns=["dimension", "time"]), column_id="my_id",
                             pivot=False,
                             default_fc_parameters=MinimalFCParameters())
        X = pd.DataFrame(X, columns=["my_id", "variable", "value"])
        self.assertIn("value__mean", X["variable"].values)
        self.assertAlmostEqual(X[(X["my_id"] == "5") & (X["variable"] == "value__mean")]["value"].iloc[0], 5.516e-05, 4)
        self.assertEqual(X.shape, (100*8, 3))

    def test_dask(self):
        df = dd.from_pandas(self.df, npartitions=1)

        X = extract_features(df, column_id="my_id", column_sort="time",
                             column_kind="dimension", column_value="value",
                             default_fc_parameters=MinimalFCParameters()).compute()
        self.assertIn("1__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "1__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 16))

        X = extract_features(df, column_id="my_id", column_sort="time",
                             column_kind="dimension",
                             default_fc_parameters=MinimalFCParameters()).compute()
        self.assertIn("1__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "1__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 16))

        X = extract_features(df.drop(columns=["dimension"]), column_id="my_id",
                             column_sort="time",
                             default_fc_parameters=MinimalFCParameters()).compute()
        self.assertIn("value__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "value__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 8))

        X = extract_features(df.drop(columns=["dimension", "time"]), column_id="my_id",
                             default_fc_parameters=MinimalFCParameters()).compute()
        self.assertIn("value__mean", X.columns)
        self.assertAlmostEqual(X.loc["5", "value__mean"], 5.516e-05, 4)
        self.assertIn("11", X.index)
        self.assertEqual(X.shape, (100, 8))

    def test_dask_no_pivot(self):
        df = dd.from_pandas(self.df, npartitions=1)

        X = extract_features(df, column_id="my_id", column_sort="time",
                             column_kind="dimension", column_value="value",
                             pivot=False,
                             default_fc_parameters=MinimalFCParameters()).compute()
        self.assertIn("1__mean", X["variable"].values)
        self.assertAlmostEqual(X[(X["my_id"] == "5") & (X["variable"] == "1__mean")]["value"].iloc[0], 5.516e-05, 4)
        self.assertEqual(X.shape, (100*16, 3))

        X = extract_features(df, column_id="my_id", column_sort="time",
                             column_kind="dimension",
                             pivot=False,
                             default_fc_parameters=MinimalFCParameters()).compute()
        self.assertIn("1__mean", X["variable"].values)
        self.assertAlmostEqual(X[(X["my_id"] == "5") & (X["variable"] == "1__mean")]["value"].iloc[0], 5.516e-05, 4)
        self.assertEqual(X.shape, (100*16, 3))

        X = extract_features(df.drop(columns=["dimension"]), column_id="my_id",
                             column_sort="time",
                             pivot=False,
                             default_fc_parameters=MinimalFCParameters()).compute()
        self.assertIn("value__mean", X["variable"].values)
        self.assertAlmostEqual(X[(X["my_id"] == "5") & (X["variable"] == "value__mean")]["value"].iloc[0], 5.516e-05, 4)
        self.assertEqual(X.shape, (100*8, 3))

        X = extract_features(df.drop(columns=["dimension", "time"]), column_id="my_id",
                             pivot=False,
                             default_fc_parameters=MinimalFCParameters()).compute()
        self.assertIn("value__mean", X["variable"].values)
        self.assertAlmostEqual(X[(X["my_id"] == "5") & (X["variable"] == "value__mean")]["value"].iloc[0], 5.516e-05, 4)
        self.assertEqual(X.shape, (100*8, 3))



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

import os
import subprocess
import tempfile
import nbformat

import pytest
from unittest import TestCase

default_timeout = 900


def _notebook_run(path, timeout=default_timeout):
    """
    Execute a singular ipython notebook at path and returns the cells and outputs

    :returns (parsed nb object, execution errors)
    """

    dirname, _ = os.path.split(path)
    execproc_timeout = '--ExecutePreprocessor.timeout=%d' % timeout

    # Do not run notebook tests on Travis.  notebooks tests should only be
    # run in the local developer testing context and notebook tests often
    # cause time out failures on Travis builds see (github #409, #410)
    try:
        if os.environ['TRAVIS']:
            return [], []
    except BaseException:
        pass

    # Ensure temporary files are not auto-deleted as processes have limited
    # permissions to re-use file handles under WinNT-based operating systems.
    fname = ''
    with tempfile.NamedTemporaryFile(mode='w+t', suffix=".ipynb", delete=False) as fout:
        fname = fout.name

        args = ["jupyter", "nbconvert",
                "--to", "notebook", "--execute", execproc_timeout]
        args += ["--ExecutePreprocessor.kernel_name=python3"]
        args += ["--output", fout.name, path]
        subprocess.check_call(args)

        fout.seek(0)
        nb = nbformat.read(fout, nbformat.current_nbformat)
    os.remove(fname)

    errors = [output for cell in nb.cells if "outputs" in cell
              for output in cell["outputs"]
              if output.output_type == "error"]
    return nb, errors


@pytest.mark.skipif(os.environ.get('TEST_NOTEBOOKS') != 'y', reason="Disabled notebook testing")
class NotebooksTestCase(TestCase):
    def test_basic_example(self):
        nb, errors = _notebook_run('notebooks/examples/01 Feature Extraction and Selection.ipynb', default_timeout)
        self.assertEqual(errors, [])

    def test_pipeline_example(self):
        nb, errors = _notebook_run('notebooks/examples/02 sklearn Pipeline.ipynb', default_timeout)
        self.assertEqual(errors, [])

    def test_extraction_settings(self):
        nb, errors = _notebook_run('notebooks/examples/03 Feature Extraction Settings.ipynb', default_timeout)
        self.assertEqual(errors, [])

    def test_multiclass_selection_example(self):
        nb, errors = _notebook_run('notebooks/examples/04 Multiclass Selection Example.ipynb', default_timeout)
        self.assertEqual(errors, [])

    def test_timeseries_forecasting(self):
        nb, errors = _notebook_run('notebooks/examples/05 Timeseries Forecasting.ipynb', default_timeout)
        self.assertEqual(errors, [])

    def test_timeseries_forecasting_exprt(self):
        nb, errors = _notebook_run('notebooks/advanced/05 Timeseries Forecasting (multiple ids).ipynb', default_timeout)
        self.assertEqual(errors, [])

    def test_inspect_dft_features(self):
        nb, errors = _notebook_run('notebooks/advanced/inspect_dft_features.ipynb', default_timeout)
        self.assertEqual(errors, [])

    def test_feature_extraction_with_datetime_index(self):
        nb, errors = _notebook_run('notebooks/advanced/feature_extraction_with_datetime_index.ipynb', default_timeout)
        self.assertEqual(errors, [])

    def test_friedrich_coefficients(self):
        nb, errors = _notebook_run('notebooks/advanced/friedrich_coefficients.ipynb', default_timeout)
        self.assertEqual(errors, [])

    def test_inspect_dft_features(self):
        nb, errors = _notebook_run('notebooks/advanced/inspect_dft_features.ipynb', default_timeout)
        self.assertEqual(errors, [])

    def test_perform_PCA_on_extracted_features(self):
        nb, errors = _notebook_run('notebooks/advanced/perform-PCA-on-extracted-features.ipynb', default_timeout)
        self.assertEqual(errors, [])

    def test_visualize_benjamini_yekutieli_procedure(self):
        nb, errors = _notebook_run('notebooks/advanced/visualize-benjamini-yekutieli-procedure.ipynb', default_timeout)
        self.assertEqual(errors, [])



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016
import pytest
import numpy as np
import pandas as pd

from tsfresh.defaults import TEST_FOR_BINARY_TARGET_REAL_FEATURE
from tsfresh.feature_selection.significance_tests import target_binary_feature_binary_test, \
    target_binary_feature_real_test, target_real_feature_real_test, target_real_feature_binary_test


@pytest.fixture()
def set_random_seed():
    np.random.seed(seed=42)


@pytest.fixture()
def binary_feature(set_random_seed):
    return pd.Series(np.random.binomial(1, 0.5, 250), name="TEST")


@pytest.fixture()
def binary_target_not_related(set_random_seed):
    return pd.Series(np.random.binomial(1, 0.5, 250))


@pytest.fixture()
def real_feature(set_random_seed):
    return pd.Series(np.random.normal(0, 1, 250), name="TEST")


@pytest.fixture()
def real_target_not_related(set_random_seed):
    return pd.Series(np.random.normal(0, 1, 250))


class TestUnsignificant:
    @pytest.fixture()
    def minimal_p_value_for_unsignificant_features(self):
        return 0.05

    def test_feature_selection_target_binary_features_binary(self, minimal_p_value_for_unsignificant_features,
                                                             binary_feature,
                                                             binary_target_not_related):
        """
        Test if the p_value returned by target_binary_feature_binary_test is
        large enough for highly unsignificant features.
        """
        p_value = target_binary_feature_binary_test(binary_feature, binary_target_not_related)
        assert minimal_p_value_for_unsignificant_features < p_value

    def test_feature_selection_target_binary_features_realvalued(self, minimal_p_value_for_unsignificant_features,
                                                                 real_feature,
                                                                 binary_target_not_related):
        """
        Test if the p_value returned by target_binary_feature_binary_test is
        large enough for highly unsignificant features.
        """
        p_value = target_binary_feature_real_test(real_feature, binary_target_not_related,
                                                  TEST_FOR_BINARY_TARGET_REAL_FEATURE)
        assert minimal_p_value_for_unsignificant_features < p_value

    def test_feature_selection_target_realvalued_features_binary(self, minimal_p_value_for_unsignificant_features,
                                                                 binary_feature,
                                                                 real_target_not_related):
        """
        Test if the p_value returned by target_real_feature_binary_test is
        large enough for highly unsignificant features."""
        p_value = target_real_feature_binary_test(binary_feature, real_target_not_related)
        assert minimal_p_value_for_unsignificant_features < p_value

    def test_feature_selection_target_realvalued_features_realvalued(self, minimal_p_value_for_unsignificant_features,
                                                                     real_feature,
                                                                     real_target_not_related):
        """
        Test if the p_value returned by target_real_feature_real_test is
        large enough for highly unsignificant features.
        """
        p_value = target_real_feature_real_test(real_feature, real_target_not_related)
        assert minimal_p_value_for_unsignificant_features < p_value


class TestSignificant:
    @pytest.fixture()
    def maximal_p_value_for_significant_features(self):
        return 0.15

    def test_feature_selection_target_binary_features_binary(self, maximal_p_value_for_significant_features,
                                                             binary_feature):
        """
        Test if the p_value returned by target_binary_feature_binary_test is
        low enough for highly significant features.
        """
        y = binary_feature - pd.Series(np.random.binomial(1, 0.1, 250) + np.random.binomial(1, 0.1, 250))
        y[y == -1] = 0
        y[y == -2] = 0
        y[y == 2] = 1

        p_value = target_binary_feature_binary_test(binary_feature, y)
        assert maximal_p_value_for_significant_features > p_value

    def test_feature_selection_target_binary_features_realvalued_mann(self, maximal_p_value_for_significant_features,
                                                                      real_feature):
        """
        Test if the p_value returned by target_binary_feature_real_test is
        low enough for highly significant features.
        """
        y = pd.Series(np.ndarray(250))
        y[real_feature >= 0.3] = 1
        y[real_feature < 0.3] = 0
        y -= pd.Series(np.random.binomial(1, 0.1, 250))
        y[y == -1] = 0
        y[y == 2] = 1

        p_value = target_binary_feature_real_test(real_feature, y, TEST_FOR_BINARY_TARGET_REAL_FEATURE)
        assert maximal_p_value_for_significant_features > p_value

    def test_feature_selection_target_binary_features_realvalued_smir(self, maximal_p_value_for_significant_features,
                                                                      real_feature):
        """
        Test if the p_value returned by target_binary_feature_real_test is
        low enough for highly significant features.
        """
        y = pd.Series(np.ndarray(250))
        y[real_feature >= 0.3] = 1
        y[real_feature < 0.3] = 0
        y -= pd.Series(np.random.binomial(1, 0.2, 250))
        y[y == -1] = 0
        y[y == 2] = 1

        p_value = target_binary_feature_real_test(real_feature, y, test="smir")
        assert maximal_p_value_for_significant_features > p_value

    def test_feature_selection_target_realvalued_features_binary(self, maximal_p_value_for_significant_features,
                                                                 binary_feature):
        """
        Test if the p_value returned by target_real_feature_binary_test is
        low enough for highly significant features.
        """
        y = binary_feature * pd.Series(np.random.normal(0, 1, 250)) + pd.Series(np.random.normal(0, 0.25, 250))

        p_value = target_real_feature_binary_test(binary_feature, y)
        assert maximal_p_value_for_significant_features > p_value

    def test_feature_selection_target_realvalued_features_realvalued(self, maximal_p_value_for_significant_features,
                                                                     real_feature):
        """
        Test if the p_value returned by target_real_feature_real_test is
        low enough for highly significant features.
        """
        y = real_feature + pd.Series(np.random.normal(0, 1, 250))

        p_value = target_real_feature_real_test(real_feature, y)

        assert maximal_p_value_for_significant_features > p_value



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2017
"""
This script can be run with:


.. code-block:: bash

   python run_tsfresh.py path_to_your_csv.csv

A corresponding csv containing time series features will be
saved as features_path_to_your_csv.csv

There are a few limitations though

- Currently this only samples to first 50 values.
- Your csv must be space delimited.
- Output is saved as path_to_your_csv.features.csv

"""

import pandas as pd
import sys
from tsfresh import extract_features
import argparse
import os


def _preprocess(df):
    """
    given a DataFrame where records are stored row-wise, rearrange it
    such that records are stored column-wise.
    """

    df = df.stack()

    df.index.rename(["id", "time"], inplace=True)  # .reset_index()
    df.name = "value"
    df = df.reset_index()

    return df


def main(console_args=None):
    parser = argparse.ArgumentParser(description="Extract features from time series stored in a CSV file and "
                                                 "write them back into another CSV file. The time series in the CSV "
                                                 "file should either have one of the dataframe-formats described in "
                                                 "http://tsfresh.readthedocs.io/en/latest/text/data_formats.html, "
                                                 "which means you have to supply the --csv-with-headers flag "
                                                 "or should be in the form "
                                                 "[time series 1 values ..., time series 2 values ...] "
                                                 "where you should not add the --csv-with-headers flag. "
                                                 "The CSV is expected to be space-separated.")
    parser.add_argument("input_file_name", help="File name of the input CSV file to read in.")
    parser.add_argument("--output-file-name", help="File name of the output CSV file to write to. "
                                                   "Defaults to input_file_name.features.csv",
                        default=None)

    parser.add_argument("--column-sort", help="Column name to be used to sort the rows. "
                                              "Only available when --csv-with-headers is enabled.",
                        default=None)
    parser.add_argument("--column-kind", help="Column name where the kind column can be found."
                                              "Only available when --csv-with-headers is enabled.",
                        default=None)
    parser.add_argument("--column-value", help="Column name where the values can be found."
                                               "Only available when --csv-with-headers is enabled.",
                        default=None)
    parser.add_argument("--column-id", help="Column name where the ids can be found."
                                            "Only available when --csv-with-headers is enabled.",
                        default=None)

    parser.add_argument('--csv-with-headers', action='store_true', help="")
    print(console_args)
    args = parser.parse_args(console_args)

    if (args.column_id or args.column_kind or args.column_sort or args.column_value) and (not args.csv_with_headers):
        raise AttributeError("You can only pass in column-value, column-kind, column-id or column-sort if "
                             "--csv-with-headers is enabled.")

    if args.csv_with_headers:
        column_kind = args.column_kind
        column_sort = args.column_sort
        column_value = args.column_value
        column_id = args.column_id
        header = 0
    else:
        column_kind = None
        column_sort = "time"
        column_value = "value"
        column_id = "id"
        header = None

    # Read in CSV file
    input_file_name = args.input_file_name
    df = pd.read_csv(input_file_name, delim_whitespace=True, header=header)

    if not args.csv_with_headers:
        df = _preprocess(df)

    df_features = extract_features(df, column_kind=column_kind,
                                   column_sort=column_sort, column_value=column_value,
                                   column_id=column_id)

    # re-cast index from float to int
    df_features.index = df_features.index.astype('int')

    # write to disk
    default_out_file_name = os.path.splitext(input_file_name)[0] + '.features.csv'
    output_file_name = args.output_file_name or default_out_file_name
    df_features.to_csv(output_file_name)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

from unittest import TestCase
import numpy as np
import pandas as pd
from tsfresh.feature_extraction.extraction import extract_features
from tsfresh.feature_extraction.settings import ComprehensiveFCParameters, MinimalFCParameters, \
    EfficientFCParameters, from_columns, TimeBasedFCParameters, IndexBasedFCParameters
from tsfresh.feature_extraction import feature_calculators
from pandas.testing import assert_frame_equal


class TestSettingsObject(TestCase):
    """
    This tests the base class ComprehensiveFCParameters
    """

    def test_from_column_raises_on_wrong_column_format(self):

        self.assertRaises(TypeError, from_columns, 42)
        self.assertRaises(TypeError, from_columns, [42])
        self.assertRaises(ValueError, from_columns, ["This is not a column name"])
        self.assertRaises(ValueError, from_columns, ["This__neither"])
        self.assertRaises(ValueError, from_columns, ["This__also__not"])

    def test_from_column_correct_for_selected_columns(self):
        tsn = "TEST_TIME_SERIES"

        # Aggregate functions
        feature_names = [tsn + '__sum_values', tsn + "__median", tsn + "__length", tsn + "__sample_entropy"]

        # Aggregate functions with params
        feature_names += [tsn + '__quantile__q_10', tsn + '__quantile__q_70', tsn + '__number_peaks__n_30',
                          tsn + '__value_count__value_inf', tsn + '__value_count__value_-inf',
                          tsn + '__value_count__value_nan']

        # Apply functions
        feature_names += [tsn + '__ar_coefficient__k_20__coeff_4', tsn + '__ar_coefficient__coeff_10__k_-1']

        kind_to_fc_parameters = from_columns(feature_names)
        self.assertCountEqual(list(kind_to_fc_parameters[tsn].keys()),
                              ["sum_values", "median", "length", "sample_entropy", "quantile", "number_peaks",
                               "ar_coefficient", "value_count"])

        self.assertIsNone(kind_to_fc_parameters[tsn]["sum_values"])
        self.assertEqual(kind_to_fc_parameters[tsn]["ar_coefficient"],
                         [{"k": 20, "coeff": 4}, {"k": -1, "coeff": 10}])

        self.assertEqual(kind_to_fc_parameters[tsn]["value_count"],
                         [{"value": np.PINF}, {"value": np.NINF}, {"value": np.NaN}])

    def test_from_column_correct_for_comprehensive_fc_parameters(self):
        fset = ComprehensiveFCParameters()
        X_org = extract_features(pd.DataFrame({"value": [1, 2, 3], "id": [1, 1, 1]}),
                                 default_fc_parameters=fset,
                                 column_id="id", column_value="value",
                                 n_jobs=0)
        inferred_fset = from_columns(X_org)
        X_new = extract_features(pd.DataFrame({"value": [1, 2, 3], "id": [1, 1, 1]}),
                                 kind_to_fc_parameters=inferred_fset,
                                 column_id="id", column_value="value",
                                 n_jobs=0)
        assert_frame_equal(X_org.sort_index(), X_new.sort_index())

    def test_from_columns_ignores_columns(self):

        tsn = "TEST_TIME_SERIES"
        feature_names = [tsn + '__sum_values', tsn + "__median", tsn + "__length", tsn + "__sample_entropy"]
        feature_names += ["THIS_COL_SHOULD_BE_IGNORED"]

        kind_to_fc_parameters = from_columns(feature_names, columns_to_ignore=["THIS_COL_SHOULD_BE_IGNORED",
                                                                               "THIS_AS_WELL"])

        self.assertCountEqual(list(kind_to_fc_parameters[tsn].keys()),
                              ["sum_values", "median", "length", "sample_entropy"])

    def test_default_calculates_all_features(self):
        """
        Test that by default a ComprehensiveFCParameters object should be set up to calculate all features defined
        in tsfresh.feature_extraction.feature_calculators
        """
        settings = ComprehensiveFCParameters()
        all_feature_calculators = [name for name, func in feature_calculators.__dict__.items()
                                   if hasattr(func, "fctype")
                                   and not hasattr(func, 'input_type')]

        for calculator in all_feature_calculators:
            self.assertIn(calculator, settings,
                          msg='Default ComprehensiveFCParameters object does not setup calculation of {}'
                          .format(calculator))

    def test_from_columns_correct_for_different_kind_datatypes(self):
        """The `settings.from_columns()` function is supposed to save the feature extraction / selection results so it
        can be reused later. It works by parsing the column names of the extracted dataframes. An unfortunate side
        effect of this is that when used with the 'long' format time series input, the typing information about the
        'kind' column is lost. For example, even if the 'kind' values are in int32, in the resulting settings dict, the
        type of the top level keys (representing different kind values) will be str
        """
        df = pd.DataFrame({'id': [1, 1, 1, 1],
                           'time': [1, 1, 2, 2],
                           'kind': [1, 2, 1, 2],
                           'value': [1, 2, 3, 4]})

        features = extract_features(df, column_id='id', column_sort='time', column_kind='kind', column_value='value',
                                    default_fc_parameters=MinimalFCParameters())
        sample_settings = from_columns(features)
        X = extract_features(df, column_id='id', column_sort='time', column_kind='kind', column_value='value',
                             kind_to_fc_parameters=sample_settings)
        assert X.shape == (1, 2 * len(MinimalFCParameters()))


class TestEfficientFCParameters(TestCase):
    """
    This tests the EfficientFCParameters( class
    """

    def test_extraction_runs_through(self):
        rfs = EfficientFCParameters()
        data = pd.DataFrame([[0, 0, 0, 0], [1, 0, 0, 0]], columns=["id", "time", "kind", "value"])

        extracted_features = extract_features(data, default_fc_parameters=rfs,
                                              column_kind="kind", column_value="value",
                                              column_sort="time", column_id="id")

        self.assertCountEqual(extracted_features.index, [0, 1])

    def test_contains_all_non_high_comp_cost_features(self):
        """
        Test that by default a EfficientFCParameters object should be set up to calculate all features defined
        in tsfresh.feature_extraction.feature_calculators that do not have the attribute "high_comp_cost"
        """
        rfs = EfficientFCParameters()
        all_feature_calculators = [name for name, func in feature_calculators.__dict__.items()
                                   if hasattr(func, "fctype") and not hasattr(func, "high_comp_cost")]

        for calculator in all_feature_calculators:
            self.assertIn(calculator, rfs,
                          msg='Default EfficientFCParameters object does not setup calculation of {}'
                          .format(calculator))

    def test_contains_all_time_based_features(self):
        """
        Test that by default a TimeBasedFCParameters object should be set up to calculate all
        features defined in tsfresh.feature_extraction.feature_calculators that have the
        attribute "index_type" == pd.DatetimeIndex
        """
        rfs = TimeBasedFCParameters()
        all_feature_calculators = [name for name, func in feature_calculators.__dict__.items()
                                   if not getattr(func, "index_type", False) != pd.DatetimeIndex]

        for calculator in all_feature_calculators:
            self.assertIn(calculator, rfs,
                          msg='Default TimeBasedFCParameters object does not setup calculation of {}'
                          .format(calculator))

    def test_contains_all_index_based_features(self):
        """
        Test that by default a IndexBasedFCParameters object should be set up to calculate all
        features defined in tsfresh.feature_extraction.feature_calculators that have the
        attribute "input" == "pd.Series"
        """
        rfs = IndexBasedFCParameters()
        all_feature_calculators = [name for name, func in feature_calculators.__dict__.items()
                                   if getattr(func, "input", None) == "pd.Series"]

        for calculator in all_feature_calculators:
            self.assertIn(calculator, rfs,
                          msg='Default IndexBasedFCParameters object does not setup calculation '
                              'of {}'.format(calculator))


class TestMinimalSettingsObject(TestCase):
    def test_all_minimal_features_in(self):
        mfs = MinimalFCParameters()

        self.assertIn("mean", mfs)
        self.assertIn("median", mfs)
        self.assertIn("minimum", mfs)
        self.assertIn("maximum", mfs)
        self.assertIn("length", mfs)
        self.assertIn("sum_values", mfs)
        self.assertIn("standard_deviation", mfs)
        self.assertIn("variance", mfs)
        self.assertNotIn("fft_coefficient", mfs)

    def test_extraction_runs_through(self):
        mfs = MinimalFCParameters()

        data = pd.DataFrame([[0, 0, 0, 0], [1, 0, 0, 0]], columns=["id", "time", "kind", "value"])

        extracted_features = extract_features(data, default_fc_parameters=mfs,
                                              column_kind="kind", column_value="value",
                                              column_sort="time", column_id="id")

        self.assertCountEqual(extracted_features.columns, ["0__median", "0__standard_deviation", "0__sum_values",
                                                           "0__maximum", "0__variance", "0__minimum", "0__mean",
                                                           "0__length"])
        self.assertCountEqual(extracted_features.index, [0, 1])



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

# Thanks to Andreas W. Kempa-Liehr for providing this snippet

import pandas as pd
import numpy as np
import logging

_logger = logging.getLogger(__name__)


class velocity:
    """
    Simulates the velocity of a dissipative soliton (kind of self organized particle) [6]_.
    The equilibrium velocity without noise R=0 for
    $\tau>1.0/\\kappa_3$ is $\\kappa_3 \\sqrt{(tau - 1.0/\\kappa_3)/Q}.
    Before the drift-bifurcation $\tau \\le 1.0/\\kappa_3$ the velocity is zero.

    References
    ----------

    .. [6] Andreas Kempa-Liehr (2013, p. 159-170)
        Dynamics of Dissipative Soliton
        Dissipative Solitons in Reaction Diffusion Systems.
        Springer: Berlin


    >>> ds = velocity(tau=3.5) # Dissipative soliton with equilibrium velocity 1.5e-3
    >>> print(ds.label) # Discriminating before or beyond Drift-Bifurcation
    1

    # Equilibrium velocity
    >>> print(ds.deterministic)
    0.0015191090506254991

    # Simulated velocity as a time series with 20000 time steps being disturbed by Gaussian white noise
    >>> v = ds.simulate(20000)
    """

    def __init__(self, tau=3.8, kappa_3=0.3, Q=1950.0, R=3e-4, delta_t=0.05, seed=None):
        """
        :param tau: Bifurcation parameter determining the intrinsic velocity of the dissipative soliton,
                    which is zero for tau<=1.0/kappa_3 and np.sqrt(kappa_3**3/Q * (tau - 1.0/kappa_3)) otherwise
        :type tau: float
        :param kappa_3: Inverse bifurcation point.
        :type kappa_3:
        :param Q: Shape parameter of dissipative soliton
        :type Q: float
        :param R: Noise amplitude
        :type R: float
        :param delta_t: temporal discretization
        :type delta_t: float
        """
        # done: add start seed

        self.delta_t = delta_t
        self.kappa_3 = kappa_3
        self.Q = Q
        self.tau = tau
        self.a = self.delta_t * kappa_3 ** 2 * (tau - 1.0 / kappa_3)
        self.b = self.delta_t * Q / kappa_3
        self.label = int(tau > 1.0 / kappa_3)
        self.c = np.sqrt(self.delta_t) * R
        self.delta_t = self.delta_t

        if seed is not None:
            np.random.seed(seed)

        if tau <= 1.0 / kappa_3:
            self.deterministic = 0.0
        else:
            self.deterministic = kappa_3 ** 1.5 * np.sqrt((tau - 1.0 / kappa_3) / Q)

    def __call__(self, v):
        """
        returns deterministic dynamic = acceleration (without noise)

        :param v: initial velocity vector
        :rtype v: ndarray
        :return: velocity vector of next time step
        :return type: ndarray
        """

        return v * (1.0 + self.a - self.b * np.dot(v, v))

    def simulate(self, N, v0=np.zeros(2)):
        """

        :param N: number of time steps
        :type N: int
        :param v0: initial velocity vector
        :type v0: ndarray
        :return: time series of velocity vectors with shape (N, v0.shape[0])
        :rtype: ndarray
        """

        v = [v0]  # first value is initial condition
        n = N - 1  # Because we are returning the initial condition,
        # only (N-1) time steps are computed
        gamma = np.random.randn(n, v0.size)
        for i in range(n):
            next_v = self.__call__(v[i]) + self.c * gamma[i]
            v.append(next_v)
        v_vec = np.array(v)
        return v_vec


def sample_tau(n=10, kappa_3=0.3, ratio=0.5, rel_increase=0.15):
    """
    Return list of control parameters

    :param n: number of samples
    :type n: int
    :param kappa_3: inverse bifurcation point
    :type kappa_3: float
    :param ratio: ratio (default 0.5) of samples before and beyond drift-bifurcation
    :type ratio: float
    :param rel_increase: relative increase from bifurcation point
    :type rel_increase: float
    :return: tau. List of sampled bifurcation parameter
    :rtype tau: list
    """
    assert ratio > 0 and ratio <= 1
    assert kappa_3 > 0
    assert rel_increase > 0 and rel_increase <= 1
    tau_c = 1.0 / kappa_3

    tau_max = tau_c * (1.0 + rel_increase)
    tau = tau_c + (tau_max - tau_c) * (np.random.rand(n) - ratio)
    return tau.tolist()


def load_driftbif(n, length, m=2, classification=True, kappa_3=0.3, seed=False):
    """
    Simulates n time-series with length time steps each for the m-dimensional velocity of a dissipative soliton

    classification=True:
    target 0 means tau<=1/0.3, Dissipative Soliton with Brownian motion (purely noise driven)
    target 1 means tau> 1/0.3, Dissipative Soliton with Active Brownian motion (intrinsiv velocity with overlaid noise)

    classification=False:
    target is bifurcation parameter tau

    :param n: number of samples
    :type n: int
    :param length: length of the time series
    :type length: int
    :param m: number of spatial dimensions (default m=2) the dissipative soliton is propagating in
    :type m: int
    :param classification: distinguish between classification (default True) and regression target
    :type classification: bool
    :param kappa_3: inverse bifurcation parameter (default 0.3)
    :type kappa_3: float
    :param seed: random seed (default False)
    :type seed: float
    :return: X, y. Time series container and target vector
    :rtype X: pandas.DataFrame
    :rtype y: pandas.DataFrame
    """

    # todo: add ratio of classes

    if m > 2:
        logging.warning("You set the dimension parameter for the dissipative soliton to m={}, however it is only"
                        "properly defined for m=1 or m=2.".format(m))

    id = np.repeat(range(n), length * m)
    dimensions = list(np.repeat(range(m), length)) * n

    labels = list()
    values = list()

    ls_tau = sample_tau(n, kappa_3=kappa_3)

    for i, tau in enumerate(ls_tau):
        ds = velocity(tau=tau, kappa_3=kappa_3, seed=seed)
        if classification:
            labels.append(ds.label)
        else:
            labels.append(ds.tau)
        values.append(ds.simulate(length, v0=np.zeros(m)).transpose().flatten())
    time = np.stack([ds.delta_t * np.arange(length)] * n * m).flatten()

    df = pd.DataFrame({'id': id, "time": time, "value": np.stack(values).flatten(), "dimension": dimensions})
    y = pd.Series(labels)
    y.index = range(n)

    return df, y



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016
import pytest
from statsmodels.stats.multitest import multipletests

import pandas as pd
import numpy as np


@pytest.mark.parametrize("p_value, ind, fdr, expected",
                         [([0, 0, 0], True, 0.10, [True, True, True]),
                          ([0, 0, 0], False, 0.10, [True, True, True]),
                          ([0.1, 0.15, 0.2, 0], True, 0.20, [True, True, True, True]),
                          ([0.1, 0.15, 0.2, 0], False, 0.20, [False, False, False, True]),
                          ([0.1, 0.1, 0.05], True, 0.20, [True, True, True]),
                          ([0.1, 0.11, 0.05], False, 0.20, [False, False, False]),
                          ([0.1, 0.1, 0.05], False, 0.20, [True, True, True]),
                          ([.00356, .01042, .01208, .02155, .03329, .11542], True, 0.05,
                           [True, True, True, True, True, False]),
                          ([.00356, .01042, .01208, .02155, .03329, .11542], False, 0.05,
                           [False, False, False, False, False, False]),
                          ([0.11, 0.001, 0.05], False, 0.20, [False, True, True]),
                          ])
def test_fdr_control(p_value, ind, fdr, expected):
    df = pd.DataFrame({"p_value": p_value})
    method = 'fdr_bh' if ind else 'fdr_by'
    df["relevant"] = multipletests(pvals=df.p_value, alpha=fdr, method=method)[0]
    result = df["relevant"].values
    expected = np.array(expected)
    assert (result == expected).all()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Setup file for tsfresh.

    This file was generated with PyScaffold 2.5.6, a tool that easily
    puts up a scaffold for your new Python project. Learn more under:
    http://pyscaffold.readthedocs.org/
"""

import sys
from setuptools import setup, find_packages

with open("README.md") as f:
    long_description = f.read()

with open("requirements.txt") as f:
    requirements = [line for line in f if not line.startswith("#")]

needs_sphinx = {"build_sphinx", "upload_docs"}.intersection(sys.argv)
sphinx = ["sphinx", "sphinx_rtd_theme"] if needs_sphinx else []

setup(
    use_scm_version=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    setup_requires=["setuptools_scm"] + sphinx,
    packages=find_packages(exclude=["tests.*", "tests"]),
    install_requires=requirements,
)



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016
"""
Utility functions for handling the DataFrame conversions to the internal normalized format
(see ``normalize_input_to_internal_representation``) or on how to handle ``NaN`` and ``inf`` in the DataFrames.
"""
import warnings

import numpy as np
import pandas as pd

from tsfresh import defaults
from tsfresh.utilities.distribution import MapDistributor, MultiprocessingDistributor, DistributorBaseClass


def check_for_nans_in_columns(df, columns=None):
    """
    Helper function to check for ``NaN`` in the data frame and raise a ``ValueError`` if there is one.

    :param df: the pandas DataFrame to test for NaNs
    :type df: pandas.DataFrame
    :param columns: a list of columns to test for NaNs. If left empty, all columns of the DataFrame will be tested.
    :type columns: list

    :return: None
    :rtype: None
    :raise: ``ValueError`` of ``NaNs`` are found in the DataFrame.
    """
    if columns is None:
        columns = df.columns

    if pd.isnull(df.loc[:, columns]).any().any():
        if not isinstance(columns, list):
            columns = list(columns)
        raise ValueError("Columns {} of DataFrame must not contain NaN values".format(
            df.loc[:, columns].columns[pd.isnull(df.loc[:, columns]).sum() > 0].tolist()))


def impute(df_impute):
    """
    Columnwise replaces all ``NaNs`` and ``infs`` from the DataFrame `df_impute` with average/extreme values from
    the same columns. This is done as follows: Each occurring ``inf`` or ``NaN`` in `df_impute` is replaced by

        * ``-inf`` -> ``min``
        * ``+inf`` -> ``max``
        * ``NaN`` -> ``median``

    If the column does not contain finite values at all, it is filled with zeros.

    This function modifies `df_impute` in place. After that, df_impute is guaranteed to not contain any non-finite
    values. Also, all columns will be guaranteed to be of type ``np.float64``.

    :param df_impute: DataFrame to impute
    :type df_impute: pandas.DataFrame

    :return df_impute: imputed DataFrame
    :rtype df_impute: pandas.DataFrame
    """
    col_to_max, col_to_min, col_to_median = get_range_values_per_column(df_impute)
    df_impute = impute_dataframe_range(df_impute, col_to_max, col_to_min, col_to_median)

    # Ensure a type of "np.float64"
    df_impute.astype(np.float64, copy=False)
    return df_impute


def impute_dataframe_zero(df_impute):
    """
    Replaces all ``NaNs``, ``-infs`` and ``+infs`` from the DataFrame `df_impute` with 0s.
    The `df_impute` will be modified in place. All its columns will be into converted into dtype ``np.float64``.

    :param df_impute: DataFrame to impute
    :type df_impute: pandas.DataFrame

    :return df_impute: imputed DataFrame
    :rtype df_impute: pandas.DataFrame
    """

    df_impute.replace([np.PINF, np.NINF], 0, inplace=True)
    df_impute.fillna(0, inplace=True)

    # Ensure a type of "np.float64"
    df_impute.astype(np.float64, copy=False)
    return df_impute


def impute_dataframe_range(df_impute, col_to_max, col_to_min, col_to_median):
    """
    Columnwise replaces all ``NaNs``, ``-inf`` and ``+inf`` from the DataFrame `df_impute` with average/extreme values
    from the provided dictionaries.

    This is done as follows: Each occurring ``inf`` or ``NaN`` in `df_impute` is replaced by

        * ``-inf`` -> by value in col_to_min
        * ``+inf`` -> by value in col_to_max
        * ``NaN`` -> by value in col_to_median

    If a column of df_impute is not found in the one of the dictionaries, this method will raise a ValueError.
    Also, if one of the values to replace is not finite a ValueError is returned

    This function modifies `df_impute` in place. Afterwards df_impute is
    guaranteed to not contain any non-finite values.
    Also, all columns will be guaranteed to be of type ``np.float64``.

    :param df_impute: DataFrame to impute
    :type df_impute: pandas.DataFrame
    :param col_to_max: Dictionary mapping column names to max values
    :type col_to_max: dict
    :param col_to_min: Dictionary mapping column names to min values
    :type col_to_max: dict
    :param col_to_median: Dictionary mapping column names to median values
    :type col_to_max: dict

    :return df_impute: imputed DataFrame
    :rtype df_impute: pandas.DataFrame
    :raise ValueError: if a column of df_impute is missing in col_to_max, col_to_min or col_to_median or a value
                       to replace is non finite
    """
    columns = df_impute.columns

    # Making sure col_to_median, col_to_max and col_to_min have entries for every column
    if not set(columns) <= set(col_to_median.keys()) or \
            not set(columns) <= set(col_to_max.keys()) or \
            not set(columns) <= set(col_to_min.keys()):
        raise ValueError("Some of the dictionaries col_to_median, col_to_max, col_to_min contains more or less keys "
                         "than the column names in df")

    # check if there are non finite values for the replacement
    if np.any(~np.isfinite(list(col_to_median.values()))) or \
            np.any(~np.isfinite(list(col_to_min.values()))) or \
            np.any(~np.isfinite(list(col_to_max.values()))):
        raise ValueError("Some of the dictionaries col_to_median, col_to_max, col_to_min contains non finite values "
                         "to replace")

    # Make the replacement dataframes as large as the real one
    col_to_max = pd.DataFrame([col_to_max] * len(df_impute), index=df_impute.index)
    col_to_min = pd.DataFrame([col_to_min] * len(df_impute), index=df_impute.index)
    col_to_median = pd.DataFrame([col_to_median] * len(df_impute), index=df_impute.index)

    df_impute.where(df_impute.values != np.PINF, other=col_to_max, inplace=True)
    df_impute.where(df_impute.values != np.NINF, other=col_to_min, inplace=True)
    df_impute.where(~np.isnan(df_impute.values), other=col_to_median, inplace=True)

    df_impute.astype(np.float64, copy=False)
    return df_impute


def get_range_values_per_column(df):
    """
    Retrieves the finite max, min and mean values per column in the DataFrame `df` and stores them in three
    dictionaries. Those dictionaries `col_to_max`, `col_to_min`, `col_to_median` map the columnname to the maximal,
    minimal or median value of that column.

    If a column does not contain any finite values at all, a 0 is stored instead.

    :param df: the Dataframe to get columnswise max, min and median from
    :type df: pandas.DataFrame

    :return: Dictionaries mapping column names to max, min, mean values
    :rtype: (dict, dict, dict)
    """
    data = df.values
    masked = np.ma.masked_invalid(data)
    columns = df.columns

    is_col_non_finite = masked.mask.sum(axis=0) == masked.data.shape[0]

    if np.any(is_col_non_finite):
        # We have columns that does not contain any finite value at all, so we will store 0 instead.
        warnings.warn("The columns {} did not have any finite values. Filling with zeros.".format(
            df.iloc[:, np.where(is_col_non_finite)[0]].columns.values), RuntimeWarning)

        masked.data[:, is_col_non_finite] = 0  # Set the values of the columns to 0
        masked.mask[:, is_col_non_finite] = False  # Remove the mask for this column

    # fetch max, min and median for all columns
    col_to_max = dict(zip(columns, np.max(masked, axis=0)))
    col_to_min = dict(zip(columns, np.min(masked, axis=0)))
    col_to_median = dict(zip(columns, np.ma.median(masked, axis=0)))

    return col_to_max, col_to_min, col_to_median


def restrict_input_to_index(df_or_dict, column_id, index):
    """
    Restrict df_or_dict to those ids contained in index.

    :param df_or_dict: a pandas DataFrame or a dictionary.
    :type df_or_dict: pandas.DataFrame or dict
    :param column_id: it must be present in the pandas DataFrame or in all DataFrames in the dictionary.
        It is not allowed to have NaN values in this column.
    :type column_id: basestring
    :param index: Index containing the ids
    :type index: Iterable or pandas.Series

    :return df_or_dict_restricted: the restricted df_or_dict
    :rtype df_or_dict_restricted: dict or pandas.DataFrame
    :raise: ``TypeError`` if df_or_dict is not of type dict or pandas.DataFrame
    """
    if isinstance(df_or_dict, pd.DataFrame):
        ids_in_df = set(df_or_dict[column_id])
        ids_in_index = set(index)
        present_ids = ids_in_index & ids_in_df
        if not present_ids:
            msg = "The ids of the time series container and the index of the input data X do not share any identifier!"
            raise AttributeError(msg)

        df_or_dict_restricted = df_or_dict[df_or_dict[column_id].isin(index)]
    elif isinstance(df_or_dict, dict):
        df_or_dict_restricted = {kind: restrict_input_to_index(df, column_id, index)
                                 for kind, df in df_or_dict.items()}
    else:
        raise TypeError("df_or_dict should be of type dict or pandas.DataFrame")

    return df_or_dict_restricted


def get_ids(df_or_dict, column_id):
    """
    Aggregates all ids in column_id from the time series container `

    :param df_or_dict: a pandas DataFrame or a dictionary.
    :type df_or_dict: pandas.DataFrame or dict
    :param column_id: it must be present in the pandas DataFrame or in all DataFrames in the dictionary.
        It is not allowed to have NaN values in this column.
    :type column_id: basestring

    :return: as set with all existing ids in energy_ratio_by_chunks
    :rtype: Set
    :raise: ``TypeError`` if df_or_dict is not of type dict or pandas.DataFrame
    """
    if isinstance(df_or_dict, pd.DataFrame):
        return set(df_or_dict[column_id])
    elif isinstance(df_or_dict, dict):
        return set.union(*[set(df[column_id]) for _, df in df_or_dict.items()])
    else:
        raise TypeError("df_or_dict should be of type dict or pandas.DataFrame")


def _roll_out_time_series(timeshift, grouped_data, rolling_direction, max_timeshift, min_timeshift,
                          column_sort, column_id):
    """
    Internal helper function for roll_time_series.
    This function has the task to extract the rolled forecast data frame of the number `timeshift`.
    This means it has shifted a virtual window if size `max_timeshift` (or infinite)
    `timeshift` times in the positive direction (for positive `rolling_direction`) or in negative direction
    (for negative `rolling_direction`).
    It starts counting from the first data point for each id (and kind) (or the last one for negative
    `rolling_direction`).
    The rolling happens for each `id` and `kind` separately.
    Extracted data smaller than `min_timeshift` + 1 are removed.

    Implementation note:
    Even though negative rolling direction means, we let the window shift in negative direction over the data,
    the counting of `timeshift` still happens from the first row onwards. Example:

        1   2   3   4

    If we do positive rolling, we extract the sub time series

      [ 1 ]               input parameter: timeshift=1, new id: ([id=]X,[timeshift=]1)
      [ 1   2 ]           input parameter: timeshift=2, new id: ([id=]X,[timeshift=]2)
      [ 1   2   3 ]       input parameter: timeshift=3, new id: ([id=]X,[timeshift=]3)
      [ 1   2   3   4 ]   input parameter: timeshift=4, new id: ([id=]X,[timeshift=]4)

    If we do negative rolling:

      [ 1   2   3   4 ]   input parameter: timeshift=1, new id: ([id=]X,[timeshift=]1)
          [ 2   3   4 ]   input parameter: timeshift=2, new id: ([id=]X,[timeshift=]2)
              [ 3   4 ]   input parameter: timeshift=3, new id: ([id=]X,[timeshift=]3)
                  [ 4 ]   input parameter: timeshift=4, new id: ([id=]X,[timeshift=]4)

    If you now reverse the order of the negative examples, it looks like shifting the
    window from the back (but it is implemented to start counting from the beginning).

    """
    def _f(x):
        if rolling_direction > 0:
            # For positive rolling, the right side of the window moves with `timeshift`
            shift_until = timeshift
            shift_from = max(shift_until - max_timeshift - 1, 0)

            df_temp = x.iloc[shift_from:shift_until] if shift_until <= len(x) else None
        else:
            # For negative rolling, the left side of the window moves with `timeshift`
            shift_from = max(timeshift - 1, 0)
            shift_until = shift_from + max_timeshift + 1

            df_temp = x.iloc[shift_from:shift_until]

        if df_temp is None or len(df_temp) < min_timeshift + 1:
            return

        df_temp = df_temp.copy()

        # and set the shift correctly
        if column_sort and rolling_direction > 0:
            timeshift_value = df_temp[column_sort].iloc[-1]
        elif column_sort and rolling_direction < 0:
            timeshift_value = df_temp[column_sort].iloc[0]
        else:
            timeshift_value = timeshift - 1
        # and now create new ones ids out of the old ones
        df_temp["id"] = df_temp[column_id].apply(lambda row: (row, timeshift_value))

        return df_temp

    return [grouped_data.apply(_f)]


def roll_time_series(df_or_dict, column_id, column_sort=None, column_kind=None,
                     rolling_direction=1, max_timeshift=None, min_timeshift=0,
                     chunksize=defaults.CHUNKSIZE,
                     n_jobs=defaults.N_PROCESSES, show_warnings=defaults.SHOW_WARNINGS,
                     disable_progressbar=defaults.DISABLE_PROGRESSBAR,
                     distributor=None):
    """
    This method creates sub windows of the time series. It rolls the (sorted) data frames for each kind and each id
    separately in the "time" domain (which is represented by the sort order of the sort column given by `column_sort`).

    For each rolling step, a new id is created by the scheme ({id}, {shift}), here id is the former id of
    the column and shift is the amount of "time" shifts.
    You can think of it as having a window of fixed length (the max_timeshift) moving one step at a time over
    your time series.
    Each cut-out seen by the window is a new time series with a new identifier.

    A few remarks:

     * This method will create new IDs!
     * The sign of rolling defines the direction of time rolling, a positive value means we are shifting
       the cut-out window foreward in time. The name of each new sub time series is given by the last time point.
       This means, the time series named `([id=]4,[timeshift=]5)` with a `max_timeshift` of 3 includes the data
       of the times 3, 4 and 5.
       A negative rolling direction means, you go in negative time direction over your data.
       The time series named `([id=]4,[timeshift=]5)` with `max_timeshift` of 3 would then include the data
       of the times 5, 6 and 7.
     * It is possible to shift time series of different lengths, but:
     * We assume that the time series are uniformly sampled
     * For more information, please see :ref:`forecasting-label`.

    :param df_or_dict: a pandas DataFrame or a dictionary. The required shape/form of the object depends on the rest of
        the passed arguments.
    :type df_or_dict: pandas.DataFrame or dict

    :param column_id: it must be present in the pandas DataFrame or in all DataFrames in the dictionary.
        It is not allowed to have NaN values in this column.
    :type column_id: basestring

    :param column_sort: if not None, sort the rows by this column. It is not allowed to
        have NaN values in this column. If not given, will be filled by an increasing number,
        meaning that the order of the passed dataframes are used as "time" for the time series.
    :type column_sort: basestring or None

    :param column_kind: It can only be used when passing a pandas DataFrame (the dictionary is already assumed to be
        grouped by the kind). Is must be present in the DataFrame and no NaN values are allowed.
        If the kind column is not passed, it is assumed that each column in the pandas DataFrame (except the id or
        sort column) is a possible kind.
    :type column_kind: basestring or None

    :param rolling_direction: The sign decides, if to shift our cut-out window backwards or forwards in "time".
    :type rolling_direction: int

    :param max_timeshift: If not None, the cut-out window is at maximum `max_timeshift` large. If none, it grows
         infinitely.
    :type max_timeshift: int

    :param min_timeshift: Throw away all extracted forecast windows smaller or equal than this. Must be larger
         than or equal 0.
    :type min_timeshift: int

    :param n_jobs: The number of processes to use for parallelization. If zero, no parallelization is used.
    :type n_jobs: int

    :param chunksize: How many shifts per job should be calculated.
    :type chunksize: None or int

    :param show_warnings: Show warnings during the feature extraction (needed for debugging of calculators).
    :type show_warnings: bool

    :param disable_progressbar: Do not show a progressbar while doing the calculation.
    :type disable_progressbar: bool

    :param distributor: Advanced parameter: set this to a class name that you want to use as a
             distributor. See the utilities/distribution.py for more information. Leave to None, if you want
             TSFresh to choose the best distributor.
    :type distributor: class

    :return: The rolled data frame or dictionary of data frames
    :rtype: the one from df_or_dict
    """

    if rolling_direction == 0:
        raise ValueError("Rolling direction of 0 is not possible")

    if max_timeshift is not None and max_timeshift <= 0:
        raise ValueError("max_timeshift needs to be positive!")

    if min_timeshift < 0:
        raise ValueError("min_timeshift needs to be positive or zero!")

    if isinstance(df_or_dict, dict):
        if column_kind is not None:
            raise ValueError("You passed in a dictionary and gave a column name for the kind. Both are not possible.")

        return {key: roll_time_series(df_or_dict=df_or_dict[key],
                                      column_id=column_id,
                                      column_sort=column_sort,
                                      column_kind=column_kind,
                                      rolling_direction=rolling_direction,
                                      max_timeshift=max_timeshift,
                                      min_timeshift=min_timeshift,
                                      chunksize=chunksize,
                                      n_jobs=n_jobs,
                                      show_warnings=show_warnings,
                                      disable_progressbar=disable_progressbar,
                                      distributor=distributor)
                for key in df_or_dict}

    # Now we know that this is a pandas data frame
    df = df_or_dict

    if len(df) <= 1:
        raise ValueError("Your time series container has zero or one rows!. Can not perform rolling.")

    if column_id is not None:
        if column_id not in df:
            raise AttributeError("The given column for the id is not present in the data.")
    else:
        raise ValueError("You have to set the column_id which contains the ids of the different time series")

    if column_kind is not None:
        grouper = [column_kind, column_id]
    else:
        grouper = [column_id, ]

    if column_sort is not None:
        # Require no Nans in column
        if df[column_sort].isnull().any():
            raise ValueError("You have NaN values in your sort column.")

        df = df.sort_values(column_sort)

        if df[column_sort].dtype != np.object:
            # if rolling is enabled, the data should be uniformly sampled in this column
            # Build the differences between consecutive time sort values

            differences = df.groupby(grouper)[column_sort].apply(
                lambda x: x.values[:-1] - x.values[1:])
            # Write all of them into one big list
            differences = sum(map(list, differences), [])
            # Test if all differences are the same
            if differences and min(differences) != max(differences):
                warnings.warn("Your time stamps are not uniformly sampled, which makes rolling "
                              "nonsensical in some domains.")

    # Roll the data frames if requested
    rolling_direction = np.sign(rolling_direction)

    grouped_data = df.groupby(grouper)
    prediction_steps = grouped_data.count().max().max()

    max_timeshift = max_timeshift or prediction_steps

    # Todo: not default for columns_sort to be None
    if column_sort is None:
        df["sort"] = range(df.shape[0])

    range_of_shifts = range(1, prediction_steps + 1)

    if distributor is None:
        if n_jobs == 0:
            distributor = MapDistributor(disable_progressbar=disable_progressbar,
                                         progressbar_title="Rolling")
        else:
            distributor = MultiprocessingDistributor(n_workers=n_jobs,
                                                     disable_progressbar=disable_progressbar,
                                                     progressbar_title="Rolling",
                                                     show_warnings=show_warnings)

    if not isinstance(distributor, DistributorBaseClass):
        raise ValueError("the passed distributor is not an DistributorBaseClass object")

    kwargs = {
        "grouped_data": grouped_data,
        "rolling_direction": rolling_direction,
        "max_timeshift": max_timeshift,
        "min_timeshift": min_timeshift,
        "column_sort": column_sort,
        "column_id": column_id,
    }

    shifted_chunks = distributor.map_reduce(_roll_out_time_series, data=range_of_shifts,
                                            chunk_size=chunksize, function_kwargs=kwargs)

    distributor.close()

    df_shift = pd.concat(shifted_chunks, ignore_index=True)

    return df_shift.sort_values(by=["id", column_sort or "sort"])


def make_forecasting_frame(x, kind, max_timeshift, rolling_direction):
    """
    Takes a singular time series x and constructs a DataFrame df and target vector y that can be used for a time series
    forecasting task.

    The returned df will contain, for every time stamp in x, the last max_timeshift data points as a new
    time series, such can be used to fit a time series forecasting model.

    See :ref:`forecasting-label` for a detailed description of the rolling process and how the feature matrix and target
    vector are derived.

    The returned time series container df, will contain the rolled time series as a flat data frame, the first format
    from :ref:`data-formats-label`.

    When x is a pandas.Series, the index will be used as id.

    :param x: the singular time series
    :type x: np.array or pd.Series
    :param kind: the kind of the time series
    :type kind: str
    :param rolling_direction: The sign decides, if to roll backwards (if sign is positive) or forwards in "time"
    :type rolling_direction: int
    :param max_timeshift: If not None, shift only up to max_timeshift. If None, shift as often as possible.
    :type max_timeshift: int

    :return: time series container df, target vector y
    :rtype: (pd.DataFrame, pd.Series)
    """
    n = len(x)

    if isinstance(x, pd.Series):
        t = x.index
    else:
        t = range(n)

    df = pd.DataFrame({"id": ["id"] * n,
                       "time": t,
                       "value": x,
                       "kind": kind})

    df_shift = roll_time_series(df,
                                column_id="id",
                                column_sort="time",
                                column_kind="kind",
                                rolling_direction=rolling_direction,
                                max_timeshift=max_timeshift)

    # drop the rows which should actually be predicted
    def mask_first(x):
        """
        this mask returns an array of 1s where the last entry is a 0
        """
        result = np.ones(len(x))
        result[-1] = 0
        return result

    mask = df_shift.groupby(['id'])['id'].transform(mask_first).astype(bool)
    df_shift = df_shift[mask]

    return df_shift, df["value"][1:]


def add_sub_time_series_index(df_or_dict, sub_length, column_id=None, column_sort=None, column_kind=None):
    """
    Add a column "id" which contains:

    - if column_id is None: for each kind (or if column_kind is None for the full dataframe) a new index built by
      "sub-packaging" the data in packages of length "sub_length". For example if you have data with the
      length of 11 and sub_length is 2, you will get 6 new packages: 0, 0; 1, 1; 2, 2; 3, 3; 4, 4; 5.
    - if column_id is not None: the same as before, just for each id seperately. The old column_id values are added
      to the new "id" column after a comma

    You can use this functions to turn a long measurement into sub-packages, where you want to extract features on.

    :param df_or_dict: a pandas DataFrame or a dictionary. The required shape/form of the object depends on the rest of
        the passed arguments.
    :type df_or_dict: pandas.DataFrame or dict
    :param column_id: it must be present in the pandas DataFrame or in all DataFrames in the dictionary.
        It is not allowed to have NaN values in this column.
    :type column_id: basestring or None
    :param column_sort: if not None, sort the rows by this column. It is not allowed to
        have NaN values in this column.
    :type column_sort: basestring or None
    :param column_kind: It can only be used when passing a pandas DataFrame (the dictionary is already assumed to be
        grouped by the kind). Is must be present in the DataFrame and no NaN values are allowed.
        If the kind column is not passed, it is assumed that each column in the pandas DataFrame (except the id or
        sort column) is a possible kind.
    :type column_kind: basestring or None

    :return: The data frame or dictionary of data frames with a column "id" added
    :rtype: the one from df_or_dict
    """

    if isinstance(df_or_dict, dict):
        if column_kind is not None:
            raise ValueError("You passed in a dictionary and gave a column name for the kind. Both are not possible.")

        return {key: add_sub_time_series_index(df_or_dict=df_or_dict[key],
                                               sub_length=sub_length,
                                               column_id=column_id,
                                               column_sort=column_sort,
                                               column_kind=column_kind)
                for key in df_or_dict}

    df = df_or_dict

    grouper = []

    if column_id is not None:
        grouper.append(column_id)
    if column_kind is not None:
        grouper.append(column_kind)

    def _add_id_column(df_chunk):
        chunk_length = len(df_chunk)
        last_chunk_number = chunk_length // sub_length
        reminder = chunk_length % sub_length

        indices = np.concatenate([np.repeat(np.arange(last_chunk_number), sub_length),
                                  np.repeat(last_chunk_number, reminder)])
        assert(len(indices) == chunk_length)

        if column_id:
            indices = list(zip(indices, df_chunk[column_id]))

        if column_sort:
            df_chunk = df_chunk.sort_values(column_sort)

        df_chunk["id"] = indices

        return df_chunk

    if grouper:
        df = df.groupby(grouper).apply(_add_id_column)
    else:
        df = _add_id_column(df)

    if column_sort:
        df = df.sort_values(column_sort)

    df = df.set_index(df.index.get_level_values(-1))

    return df



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

from builtins import range
from unittest import TestCase
import pandas as pd
import pandas.testing as pdt
from sklearn.exceptions import NotFittedError
import warnings

import numpy as np
import numpy.testing as npt

from tsfresh.transformers.per_column_imputer import PerColumnImputer


class PerColumnImputerTestCase(TestCase):
    def setUp(self):
        np.random.seed(0)

    def test_not_fitted(self):
        imputer = PerColumnImputer()

        X = pd.DataFrame()

        self.assertRaises(NotFittedError, imputer.transform, X)

    def test_only_nans_and_infs(self):
        imputer = PerColumnImputer()

        X = pd.DataFrame(index=list(range(100)))

        X["NaNs"] = np.nan * np.ones(100)
        X["PINF"] = np.PINF * np.ones(100)
        X["NINF"] = np.NINF * np.ones(100)

        with warnings.catch_warnings(record=True) as w:
            imputer.fit(X)
            self.assertEqual(len(w), 1)
            self.assertEqual("The columns ['NaNs' 'PINF' 'NINF'] did not have any finite values. Filling with zeros.",
                             str(w[0].message))

        selected_X = imputer.transform(X)

        self.assertTrue((selected_X.values == 0).all())

    def test_with_numpy_array(self):
        imputer = PerColumnImputer()

        X = pd.DataFrame(index=list(range(100)))

        X["NaNs"] = np.nan * np.ones(100)
        X["PINF"] = np.PINF * np.ones(100)
        X["NINF"] = np.NINF * np.ones(100)

        X_numpy = X.values.copy()

        with warnings.catch_warnings(record=True) as w:
            imputer.fit(X)
            self.assertEqual(len(w), 1)
            self.assertEqual("The columns ['NaNs' 'PINF' 'NINF'] did not have any finite values. Filling with zeros.",
                             str(w[0].message))

        selected_X = imputer.transform(X)

        # re-initialize for new dicts
        imputer = PerColumnImputer()
        with warnings.catch_warnings(record=True) as w:
            imputer.fit(X_numpy)
            self.assertEqual(len(w), 1)
            self.assertEqual("The columns [0 1 2] did not have any finite values. Filling with zeros.",
                             str(w[0].message))

        selected_X_numpy = imputer.transform(X_numpy)

        npt.assert_array_equal(selected_X.values, selected_X_numpy.values)

        self.assertTrue(selected_X_numpy.shape, (1, 100))

    def test_standard_replacement_behavior(self):
        imputer = PerColumnImputer()

        data = [np.NINF, np.PINF, np.nan, 100.0, -100.0, 1.0, 1.0]
        truth = [-100.0, 100.0, 1.0, 100.0, -100.0, 1.0, 1.0]
        X = pd.DataFrame({"a": data})
        true_X = pd.DataFrame({"a": truth})

        imputer.fit(X)
        selected_X = imputer.transform(X)

        pdt.assert_frame_equal(selected_X, true_X)

    def test_partial_preset_col_to_NINF_given(self):
        data = [np.NINF, np.PINF, np.nan, 100.0, -100.0, 1.0, 1.0]
        truth = [-100.0, 100.0, 1.0, 100.0, -100.0, 1.0, 1.0]
        X = pd.DataFrame({"a": data})
        true_X = pd.DataFrame({"a": truth})

        col_to_min = {"a": -100}
        imputer = PerColumnImputer(col_to_NINF_repl_preset=col_to_min)

        imputer.fit(X)
        selected_X = imputer.transform(X)

        pdt.assert_frame_equal(selected_X, true_X)

    def test_partial_preset_col_to_PINF_given(self):
        data = [np.NINF, np.PINF, np.nan, 100.0, -100.0, 1.0, 1.0]
        truth = [-100.0, 100.0, 1.0, 100.0, -100.0, 1.0, 1.0]
        X = pd.DataFrame({"a": data})
        true_X = pd.DataFrame({"a": truth})

        col_to_max = {"a": 100}
        imputer = PerColumnImputer(col_to_PINF_repl_preset=col_to_max)

        imputer.fit(X)
        selected_X = imputer.transform(X)

        pdt.assert_frame_equal(selected_X, true_X)

    def test_partial_preset_col_to_NAN_given(self):
        data = [np.NINF, np.PINF, np.nan, 100.0, -100.0, 1.0, 1.0]
        truth = [-100.0, 100.0, 1.0, 100.0, -100.0, 1.0, 1.0]
        X = pd.DataFrame({"a": data})
        true_X = pd.DataFrame({"a": truth})

        col_to_median = {"a": 1}
        imputer = PerColumnImputer(col_to_NAN_repl_preset=col_to_median)

        imputer.fit(X)
        selected_X = imputer.transform(X)

        pdt.assert_frame_equal(selected_X, true_X)

    def test_different_shapes_fitted_and_transformed(self):
        imputer = PerColumnImputer()

        X = pd.DataFrame(index=list(range(10)))
        X["a"] = np.ones(10)

        imputer.fit(X)
        X["b"] = np.ones(10)

        self.assertRaises(ValueError, imputer.transform, X)

    def test_preset_has_higher_priority_than_fit(self):
        data = [np.NINF, np.PINF, np.nan, 100.0, -100.0, 1.0, 1.0]
        truth = [-100.0, 100.0, 0.0, 100.0, -100.0, 1.0, 1.0]

        X = pd.DataFrame({"a": data})
        true_X = pd.DataFrame({"a": truth})

        col_to_median = {"a": 0}
        imputer = PerColumnImputer(col_to_NAN_repl_preset=col_to_median)
        imputer.fit(X)

        selected_X = imputer.transform(X)

        pdt.assert_frame_equal(selected_X, true_X)

    def test_only_parameters_of_last_fit_count(self):
        data = [np.NINF, np.PINF, np.nan, 100.0, -100.0, 1.0, 1.0]
        data_2 = [np.NINF, np.PINF, np.nan, 10.0, -10.0, 3.0, 3.0]
        truth_a = [-10.0, 10.0, 3.0, 10.0, -10.0, 3.0, 3.0]
        truth_b = [-10.0, 10.0, 3.0, 10.0, -10.0, 3.0, 3.0]

        X = pd.DataFrame({"a": data, "b": data})
        X_2 = pd.DataFrame({"a": data_2, "b": data_2})
        true_X = pd.DataFrame({"a": truth_a, "b": truth_b})

        imputer = PerColumnImputer()

        imputer.fit(X)
        imputer.fit(X_2)

        selected_X = imputer.transform(X_2)

        pdt.assert_frame_equal(selected_X, true_X)

    def test_only_subset_of_columns_given(self):
        data = [np.NINF, np.PINF, np.nan, 100.0, -100.0, 1.0, 1.0]
        truth_a = [-100.0, 100.0, 0.0, 100.0, -100.0, 1.0, 1.0]
        truth_b = [-100.0, 100.0, 1.0, 100.0, -100.0, 1.0, 1.0]
        X = pd.DataFrame({"a": data, "b": data})
        true_X = pd.DataFrame({"a": truth_a, "b": truth_b})

        col_to_median = {"a": 0}
        imputer = PerColumnImputer(col_to_NAN_repl_preset=col_to_median)

        imputer.fit(X)
        selected_X = imputer.transform(X)

        pdt.assert_frame_equal(selected_X, true_X)

    def test_NINF_preset_contains_more_columns_than_dataframe_to_fit(self):
        X = pd.DataFrame(index=list(range(10)))
        X["a"] = np.ones(10)

        col_to_min = {"a": 0, "b": 0}

        imputer = PerColumnImputer(col_to_NINF_repl_preset=col_to_min)

        self.assertRaises(ValueError, imputer.fit, X)

    def test_PINF_preset_contains_more_columns_than_dataframe_to_fit(self):
        X = pd.DataFrame(index=list(range(10)))
        X["a"] = np.ones(10)

        col_to_max = {"a": 0, "b": 0}

        imputer = PerColumnImputer(col_to_PINF_repl_preset=col_to_max)

        self.assertRaises(ValueError, imputer.fit, X)

    def test_NAN_preset_contains_more_columns_than_dataframe_to_fit(self):
        X = pd.DataFrame(index=list(range(10)))
        X["a"] = np.ones(10)

        col_to_median = {"a": 0, "b": 0}

        imputer = PerColumnImputer(col_to_NAN_repl_preset=col_to_median)

        self.assertRaises(ValueError, imputer.fit, X)



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

import pandas as pd
import numpy as np
from sklearn import model_selection
from sklearn.ensemble import RandomForestClassifier
from sklearn.pipeline import Pipeline

from tests.fixtures import DataTestCase
import mock

from tsfresh.feature_extraction import MinimalFCParameters
from tsfresh.transformers.relevant_feature_augmenter import RelevantFeatureAugmenter
from tests.fixtures import warning_free


class RelevantFeatureAugmenterTestCase(DataTestCase):
    def setUp(self):
        self.test_df = self.create_test_data_sample()
        fc_parameters = {"length": None}
        self.kind_to_fc_parameters = {"a": fc_parameters.copy(),
                                      "b": fc_parameters.copy()}

    def test_not_fitted(self):
        augmenter = RelevantFeatureAugmenter()

        X = pd.DataFrame()

        self.assertRaises(RuntimeError, augmenter.transform, X)

    def test_no_timeseries(self):
        augmenter = RelevantFeatureAugmenter()

        X = pd.DataFrame()
        y = pd.Series(dtype="float64")

        self.assertRaises(RuntimeError, augmenter.fit, X, y)

    def test_nothing_relevant(self):
        augmenter = RelevantFeatureAugmenter(kind_to_fc_parameters=self.kind_to_fc_parameters,
                                             column_value="val", column_id="id", column_sort="sort",
                                             column_kind="kind")

        y = pd.Series({10: 1, 500: 0})
        X = pd.DataFrame(index=[10, 500])

        augmenter.set_timeseries_container(self.test_df)
        augmenter.fit(X, y)

        transformed_X = augmenter.transform(X.copy())

        self.assertEqual(list(transformed_X.columns), [])
        self.assertEqual(list(transformed_X.index), list(X.index))

    def test_evaluate_only_added_features_true(self):
        """
        The boolean flag `evaluate_only_extracted_features` makes sure that only the time series based features are
        filtered. This unit tests checks that
        """

        augmenter = RelevantFeatureAugmenter(kind_to_fc_parameters=self.kind_to_fc_parameters,
                                             filter_only_tsfresh_features=True,
                                             column_value="val", column_id="id", column_sort="sort", column_kind="kind")

        y = pd.Series({10: 1, 500: 0})
        X = pd.DataFrame(index=[10, 500])
        X["pre_feature"] = 0

        augmenter.set_timeseries_container(self.test_df)
        augmenter.fit(X, y)
        transformed_X = augmenter.transform(X.copy())

        self.assertEqual(sum(["pre_feature" == column for column in transformed_X.columns]), 1)

    def test_evaluate_only_added_features_false(self):
        """
        The boolean flag `evaluate_only_extracted_features` makes sure that only the time series based features are
        filtered. This unit tests checks that
        """

        augmenter = RelevantFeatureAugmenter(kind_to_fc_parameters=self.kind_to_fc_parameters,
                                             filter_only_tsfresh_features=False,
                                             column_value="val", column_id="id", column_sort="sort", column_kind="kind")

        df, y = self.create_test_data_sample_with_target()
        X = pd.DataFrame(index=np.unique(df.id))
        X["pre_drop"] = 0
        X["pre_keep"] = y

        augmenter.set_timeseries_container(df)
        augmenter.fit(X, y)
        transformed_X = augmenter.transform(X.copy())

        self.assertEqual(sum(["pre_keep" == column for column in transformed_X.columns]), 1)
        self.assertEqual(sum(["pre_drop" == column for column in transformed_X.columns]), 0)

    @mock.patch('tsfresh.transformers.feature_selector.calculate_relevance_table')
    def test_does_impute(self, calculate_relevance_table_mock):
        df = pd.DataFrame([[1, 1, 1], [2, 1, 1]], columns=['id', 'time', 'value'])
        X = pd.DataFrame(index=[1])
        y = pd.Series([0, 1])
        fc_parameters = {"autocorrelation": [{'lag': 2}]}

        calculate_relevance_table_mock.return_value = pd.DataFrame(columns=['feature', 'p_value', 'relevant'])
        augmenter = RelevantFeatureAugmenter(column_id='id', column_sort='time', default_fc_parameters=fc_parameters)
        augmenter.set_timeseries_container(df)
        with warning_free():
            augmenter.fit(X, y)

        assert calculate_relevance_table_mock.call_count == 1
        assert not calculate_relevance_table_mock.call_args[0][0].isnull().any().any()

    def test_no_ids_present(self):
        augmenter = RelevantFeatureAugmenter(kind_to_fc_parameters=self.kind_to_fc_parameters,
                                             filter_only_tsfresh_features=False,
                                             column_value="val", column_id="id", column_sort="sort", column_kind="kind")

        df, y = self.create_test_data_sample_with_target()
        X_with_wrong_ids = pd.DataFrame(index=[-999])

        augmenter.set_timeseries_container(df)

        self.assertRaisesRegex(AttributeError, r"The ids of the time series container",
                               augmenter.fit, X_with_wrong_ids, y)


def test_relevant_augmentor_cross_validated():
    """Validates that the RelevantFeatureAugmenter can cloned in pipelines, see issue 537
    """
    n = 16  # number of samples, needs to be divisable by 4
    index = range(n)
    df_ts = pd.DataFrame({"time": [10, 11] * n, "id": np.repeat(index, 2),
                          "value": [0, 1] * (n // 4) + [1, 2] * (n // 4) +  # class 0
                                   [10, 11] * (n // 4) + [12, 14] * (n // 4)})
    y = pd.Series(data=[0] * (n // 2) + [1] * (n // 2), index=index)
    X = pd.DataFrame(index=index)
    augmenter = RelevantFeatureAugmenter(column_id='id', column_sort='time', timeseries_container=df_ts,
                                         default_fc_parameters=MinimalFCParameters(),
                                         disable_progressbar=True, show_warnings=False, fdr_level=0.90)
    pipeline = Pipeline([('augmenter', augmenter),
                         ('classifier', RandomForestClassifier(random_state=1))])

    scores = model_selection.cross_val_score(pipeline, X, y, cv=2)
    assert (scores == np.array([1, 1])).all()



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

from unittest import TestCase
from tsfresh.examples.har_dataset import download_har_dataset, load_har_dataset, load_har_classes
from pandas import DataFrame, Series
import tempfile
import shutil


class HumanActivityTestCase(TestCase):
    def setUp(self):
        self.temporary_folder = tempfile.mkdtemp()

        download_har_dataset(folder_name=self.temporary_folder)
        self.data = load_har_dataset(folder_name=self.temporary_folder)
        self.classes = load_har_classes(folder_name=self.temporary_folder)

    def tearDown(self):
        shutil.rmtree(self.temporary_folder)

    def test_characteristics_downloaded_robot_execution_failures(self):
        self.assertEqual(len(self.data), 7352)
        self.assertIsInstance(self.data, DataFrame)

        self.assertEqual(len(self.classes), 7352)
        self.assertIsInstance(self.classes, Series)

    def test_index(self):
        self.assertCountEqual(self.data.index, self.classes.index)
